<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Criptopayer for Elementor
 * Customizable crypto donates for Elementor editor
 * Exclusively on https://1.envato.market/criptopayer-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.0
 * @copyright       (C) 2018 - 2022 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\CriptopayerElementor;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Css_Filter;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Elementor\Plugin;
use Elementor\Repeater;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\CriptopayerElementor\Unity\Settings;
use Merkulove\CriptopayerElementor\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Criptopayer - Custom Elementor Widget.
 **/
class criptopayer_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
                'mdp-criptopayer-elementor-admin',
                  UnityPlugin::get_url() . 'src/Merkulove/Unity/assets/css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
                     [],
                     UnityPlugin::get_version()
        );
        wp_register_style(
                'mdp-criptopayer-elementor',
                   UnityPlugin::get_url() . 'css/criptopayer-elementor' . UnityPlugin::get_suffix() . '.css',
                      [],
                      UnityPlugin::get_version()
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-criptopayer-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Criptopayer', 'criptopayer-elementor' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-criptopayer-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'general' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Criptopayer' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-criptopayer-elementor', 'mdp-criptopayer-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-criptopayer' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Donation Button Content Section. */
        $this->section_content_donation_button();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Button. */
        $this->section_style_button();

    }

    /**
     * Add widget controls: Content -> Button Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_donation_button() {

        $this->start_controls_section( 'section_content_donation_button', [
            'label' => esc_html__( 'Button', 'criptopayer-elementor' ),
            'tab' => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_responsive_control(
            'donation_button_alignment',
            [
                'label' => esc_html__( 'Alignment', 'criptopayer-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'default' => 'left',
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'criptopayer-elementor' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'criptopayer-elementor' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'criptopayer-elementor' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-criptopayer-elementor-box' => 'justify-content: {{VALUE}};',
                ],
                'toggle' => true,
            ]
        );

        $this->add_responsive_control(
            'donation_button_text_alignment',
            [
                'label' => esc_html__( 'Button text alignment', 'criptopayer-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'default' => 'center',
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'criptopayer-elementor' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'criptopayer-elementor' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'criptopayer-elementor' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-criptopayer-elementor-donation-button' => 'justify-content: {{VALUE}};',
                ],
                'toggle' => true,
            ]
        );

        $this->add_control(
            'button_content',
            [
                'label' => esc_html__( 'Button content', 'criptopayer-elementor' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Donate', 'criptopayer-elementor' ),
                'placeholder' => esc_html__( 'Type your button content here', 'criptopayer-elementor' ),
            ]
        );

        $this->add_control(
            'enable_button_icon',
            [
                'label' => esc_html__( 'Enable button icon', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'criptopayer-elementor' ),
                'label_off' => esc_html__( 'No', 'criptopayer-elementor' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'button_icon',
            [
                'label' => esc_html__( 'Button icon', 'criptopayer-elementor' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-star',
                    'library' => 'solid',
                ],
                'condition' => [
                    'enable_button_icon' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'button_icon_position',
            [
                'label' => esc_html__( 'Icon position', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'before',
                'options' => [
                    'before'  => esc_html__( 'Left', 'criptopayer-elementor' ),
                    'after' => esc_html__( 'Right', 'criptopayer-elementor' ),
                    'above-left' => esc_html__( 'Top left', 'criptopayer-elementor' ),
                    'above-center' => esc_html__( 'Top center', 'criptopayer-elementor' ),
                    'above-right' => esc_html__( 'Top right', 'criptopayer-elementor' ),
                    'under-left' => esc_html__( 'Bottom left', 'criptopayer-elementor' ),
                    'under-center' => esc_html__( 'Bottom center', 'criptopayer-elementor' ),
                    'under-right' => esc_html__( 'Bottom right', 'criptopayer-elementor' )
                ],
                'condition' => [
                    'enable_button_icon' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'button_hover_animation',
            [
                'label' => esc_html__( 'Hover Animation', 'criptopayer-elementor' ),
                'type' => Controls_Manager::HOVER_ANIMATION,
            ]
        );
        
        $this->add_control(
            'button_cursor_type',
            [
                'label' => esc_html__( 'Cursor type', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SELECT2,
                'separator' => 'before',
                'label_block' => true,
                'default' => 'pointer',
                'options' => [
                    'auto'       => esc_html__( 'Default', 'criptopayer-elementor' ),
                    'context-menu'  => esc_html__( 'Context menu', 'criptopayer-elementor' ),
                    'help'          => esc_html__( 'Help', 'criptopayer-elementor' ),
                    'pointer'       => esc_html__( 'Pointer', 'criptopayer-elementor' ),
                    'progress'      => esc_html__( 'Progress', 'criptopayer-elementor' ),
                    'wait'          => esc_html__( 'Wait', 'criptopayer-elementor' ),
                    'cell'          => esc_html__( 'Cell', 'criptopayer-elementor' ),
                    'crosshair'     => esc_html__( 'Crosshair', 'criptopayer-elementor' ),
                    'text'          => esc_html__( 'Text', 'criptopayer-elementor' ),
                    'vertical-text' => esc_html__( 'Vertical text', 'criptopayer-elementor' ),
                    'alias'         => esc_html__( 'Alias', 'criptopayer-elementor' ),
                    'copy'          => esc_html__( 'Copy', 'criptopayer-elementor' ),
                    'move'          => esc_html__( 'Move', 'criptopayer-elementor' ),
                    'no-drop'       => esc_html__( 'No drop', 'criptopayer-elementor' ),
                    'not-allowed'   => esc_html__( 'Not allowed	', 'criptopayer-elementor' ),
                    'all-scroll'    => esc_html__( 'All scroll', 'criptopayer-elementor' ),
                    'col-resize'    => esc_html__( 'Col resize', 'criptopayer-elementor' ),
                    'row-resize'    => esc_html__( 'Row resize', 'criptopayer-elementor' ),
                    'n-resize'      => esc_html__( 'N resize', 'criptopayer-elementor' ),
                    'ne-resize'     => esc_html__( 'Ne resize', 'criptopayer-elementor' ),
                    'e-resize'      => esc_html__( 'E resize', 'criptopayer-elementor' ),
                    'se-resize'     => esc_html__( 'Se resize', 'criptopayer-elementor' ),
                    's-resize'      => esc_html__( 'S resize', 'criptopayer-elementor' ),
                    'sw-resize'     => esc_html__( 'Sw resize', 'criptopayer-elementor' ),
                    'w-resize'      => esc_html__( 'W resize', 'criptopayer-elementor' ),
                    'nw-resize'     => esc_html__( 'Nw resize', 'criptopayer-elementor' ),
                    'nesw-resize'   => esc_html__( 'Nesw resize', 'criptopayer-elementor' ),
                    'nwse-resize'   => esc_html__( 'Nwse resize', 'criptopayer-elementor' ),
                    'zoom-in'       => esc_html__( 'Zoom in', 'criptopayer-elementor' ),
                    'zoom-out'      => esc_html__( 'Zoom out', 'criptopayer-elementor' ),
                    'grab'          => esc_html__( 'Grab', 'criptopayer-elementor' ),
                    'grabbing'      => esc_html__( 'Grabbing', 'criptopayer-elementor' ),
                    'custom'        => esc_html__( 'Custom', 'criptopayer-elementor' ),
                ],
            ]
        );

        /** Cursor Image. */
        $this->add_control(
            'button_image',
            [
                'label' => esc_html__( 'Cursor image', 'criptopayer-elementor' ),
                'type' => Controls_Manager::MEDIA,
                'condition' => [ 'button_cursor_type' => 'custom'],
                'description' => esc_html__( 'Select an image(png) with transparent background and not more 120x120px size.','criptopayer-elementor' ),
            ]
        );

        $this->end_controls_section();

    }

    /**
     *  generate box shadow style.
     *
     * @param $settings - Settings for setting styles.
     * @param $type - Type of event.
     *
     * @return string
     */
    private function get_box_shadow_style( $settings, $type ) {

        $box_shadow = 'box-shadow: ';

        foreach ( $settings[$type . '_neumorphism'] as $val ) {

            /** Is this inset shadow? */
            $inset = '';
            if ( 'yes' === $val[$type . '_inner_shadow'] ) { $inset = 'inset '; }

            $box_shadow .=
                esc_attr__( $inset ) . esc_attr__( $val[ $type . '_x_offset']['size'] ) . esc_attr__( $val[ $type . '_x_offset']['unit'] ) . ' '
                . esc_attr__( $val[$type . '_y_offset']['size'] ) . esc_attr__( $val[ $type . '_y_offset']['unit'] ) . ' '
                . esc_attr__( $val[$type . '_blur_radius']['size'] ) . esc_attr__( $val[ $type . '_blur_radius']['unit'] ) . ' '
                . esc_attr__( $val[$type . '_spread_radius']['size'] ) . esc_attr__( $val[ $type . '_spread_radius']['unit'] ). ' '
                . esc_attr__( $val[$type . '_box_shadow_color'] );

            /** Combine multiple shadow into one css rule. */
            if ( next( $settings[$type . '_neumorphism'] ) ) {

                $box_shadow .= ", ";

            } else {

                $box_shadow .= ";";

            }

        }

        return $box_shadow;

    }

    /**
     *  generate gradient style.
     *
     * @param $settings - Settings for setting styles.
     * @param $type - Type of event.
     *
     * @return string
     */
    private function get_gradient_style( $settings, $type ) {
        $background = $settings[$type . '_gradient_type'] === 'linear' ?
            'background: linear-gradient('.$settings[$type . '_gradient_angle']['size'] . 'deg,' :
            'background: radial-gradient(at '. str_replace( '-' , ' ', $settings[$type . '_gradient_position'] ) . ', ';

        foreach ( $settings[ $type . '_multigradient'] as $value ) {
            $background .= ''.$value[$type . '_gradient_color']
                .' '.$value[$type . '_gradient_location']['size'].'%';

            if ( next( $settings[$type . '_multigradient'] ) ) {
                $background .= ', ';
            } else {
                $background .= ');';
            }
        }

        return $background;
    }

    /**
     * Render CSS for each instance of criptopayer widget.
     *
     * @param array $settings
     *
     * @return void
     * @since  1.0.0
     * @access private
     *
     */
    private function render_css( $settings ) {

        $css = "";

        /** Get unique widget id. */
        $id = $this->get_id();

        /** Prepare cursor value. */
        $cursor_type = $settings['button_cursor_type'];
        if ( 'custom' === $settings['button_cursor_type'] ) {

            $url = $settings['button_image']['url'];
            $cursor_type = "url( {$url} ), auto";
        }

        /** Styles for the cursor. */
        // language=CSS
        $css .= ".mdp-criptopayer-elementor-wrapper-{$id} .mdp-criptopayer-elementor-donation-button { cursor: {$cursor_type}; } ";

        /** Normal. */
        if ( ! empty( $settings['normal_neumorphism'] ) ) {

            /** Get shadow values. */
            $shadows = $this->get_box_shadow_style( $settings, 'normal' );

            // language=CSS
            $css .= ".mdp-criptopayer-elementor-wrapper-{$id} .mdp-criptopayer-elementor-donation-button { {$shadows} }";

        }

        /** Hover. */
        if ( ! empty( $settings['hover_neumorphism'] ) ) {

            /** Get shadow values. */
            $shadows = $this->get_box_shadow_style( $settings, 'hover' );

            // language=CSS
            $css .= ".mdp-criptopayer-elementor-wrapper-{$id} .mdp-criptopayer-elementor-donation-button:hover { {$shadows} }";

        }

        if ( !empty( $settings['normal_multigradient'] ) ) {
            $backgrounds = $this->get_gradient_style( $settings, 'normal' );

            $css .= ".mdp-criptopayer-elementor-wrapper-{$id} .mdp-criptopayer-elementor-donation-button { {$backgrounds} }";

        }

        if ( !empty( $settings['hover_multigradient'] ) ) {
            $backgrounds = $this->get_gradient_style( $settings, 'hover' );

            $css .= ".mdp-criptopayer-elementor-wrapper-{$id} .mdp-criptopayer-elementor-donation-button:hover { {$backgrounds} }";

        }


        echo wp_sprintf( '<style>%s</style>', $css ); // XSS ok.

    }

    /**
     * Register controls for multi-gradient controls in the Style Tab
     *
     * @param string $key - Key of the button style tab ( normal, hover, active )
     *
     * @since 1.0.0
     * @access private
     *
     * @return void with category names
     */
    private function gradient_controls( $key ) {
        $repeater = new Repeater();

        /** Color */
        $repeater->add_control(
            $key . '_gradient_color',
            [
                'label' => esc_html__( 'Color', 'criptopayer-elementor' ),
                'type' => Controls_Manager::COLOR,
            ]
        );

        $repeater->add_control(
            $key . '_gradient_location',
            [
                'label' => esc_html__( 'Location', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 50,
                ],
            ]
        );

        /** Multi-gradient */
        $this->add_control(
            $key . '_multigradient',
            [
                'label' => esc_html__( 'Multi-gradient', 'criptopayer-elementor' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'separator' => 'before',
                'prevent_empty' => false,
                'condition' => [
                    $key . '_enable_multigradient' => 'yes'
                ]
            ]
        );

        $this->add_control(
            $key . '_gradient_type',
            [
                'label' => esc_html__( 'Type', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'linear',
                'options' => [
                    'linear'  => esc_html__( 'Linear', 'criptopayer-elementor' ),
                    'radial' => esc_html__( 'Radial', 'criptopayer-elementor' ),
                ],
                'condition' => [
                    $key . '_enable_multigradient' => 'yes'
                ]
            ]
        );

        $this->add_control(
            $key . '_gradient_angle',
            [
                'label' => esc_html__( 'Angle', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 360,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 20,
                ],
                'condition' => [
                    $key . '_gradient_type' => 'linear',
                    $key . '_enable_multigradient' => 'yes'
                ]
            ]
        );

        $this->add_control(
            $key . '_gradient_position',
            [
                'label' => esc_html__( 'Position', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'center-center',
                'options' => [
                    'center-center'  => esc_html__( 'Center Center', 'criptopayer-elementor' ),
                    'center-left' => esc_html__( 'Center Left', 'criptopayer-elementor' ),
                    'center-right' => esc_html__( 'Center Right', 'criptopayer-elementor' ),
                    'top-center' => esc_html__( 'Top Center', 'criptopayer-elementor' ),
                    'top-left' => esc_html__( 'Top Left', 'criptopayer-elementor' ),
                    'top-right' => esc_html__( 'Top Right', 'criptopayer-elementor' ),
                    'bottom-center' => esc_html__( 'Bottom Center', 'criptopayer-elementor' ),
                    'bottom-left' => esc_html__( 'Bottom Left', 'criptopayer-elementor' ),
                    'bottom-right' => esc_html__( 'Bottom Right', 'criptopayer-elementor' )
                ],
                'condition' => [
                    $key . '_gradient_type' => 'radial',
                    $key . '_enable_multigradient' => 'yes'
                ]
            ]
        );
    }

    /**
     * Register controls for multi-shadow controls in the Style Tab
     *
     * @param string $key - Key of the button style tab ( normal, hover, active )
     *
     * @since 1.0.0
     * @access protected
     *
     * @return void with category names
     */
    private function multishadow_controls( $key ) {

        /** @var Repeater Variable $repeater */
        $repeater = new Repeater();

        /** Color */
        $repeater->add_control(
            $key . '_box_shadow_color',
            [
                'label' => esc_html__( 'Color', 'criptopayer-elementor' ),
                'type' => Controls_Manager::COLOR,
            ]
        );

        /** Shadow Horizontal */
        $repeater->add_control(
            $key . '_x_offset',
            [
                'label' => esc_html__( 'Horizontal', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -1000,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 0,
                ],
            ]
        );

        /** Shadow Vertical */
        $repeater->add_control(
            $key . '_y_offset',
            [
                'label' => esc_html__( 'Vertical', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -1000,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 0,
                ],
            ]
        );

        /** Shadow Blur Radius. */
        $repeater->add_control(
            $key . '_blur_radius',
            [
                'label' => esc_html__( 'Blur', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -1000,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 10,
                ],
            ]
        );

        /** Shadow Spread Radius. */
        $repeater->add_control(
            $key . '_spread_radius',
            [
                'label' => esc_html__( 'Spread', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -1000,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 0,
                ],
            ]
        );

        /** Inner shadow */
        $repeater->add_control(
            $key . '_inner_shadow',
            [
                'label' => esc_html__( 'Inner shadow', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
            ]
        );

        /** Multi-shadow */
        $this->add_control(
            $key . '_neumorphism',
            [
                'label' => esc_html__( 'Multi-shadow', 'criptopayer-elementor' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'separator' => 'before',
                'prevent_empty' => false
            ]
        );

    }

    /**
     * Add widget controls: Style -> Section Style Button.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_button() {

        $this->start_controls_section( 'section_style_button', [
            'label' => esc_html__( 'Button', 'criptopayer-elementor' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'section_style_button_margin',
            [
                'label' => esc_html__( 'Margin', 'criptopayer-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .mdp-criptopayer-elementor-donation-button" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'section_style_button_padding',
            [
                'label' => esc_html__( 'Padding', 'criptopayer-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .mdp-criptopayer-elementor-donation-button" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'icon_size_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_responsive_control(
            'section_style_button_icon_spacing',
            [
                'label' => esc_html__( 'Icon spacing', 'criptopayer-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'condition' => [
                    'enable_button_icon' => 'yes'
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-criptopayer-elementor-btn-icon" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'button_icon_size',
            [
                'label' => esc_html__( 'Icon size', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'condition' => [
                    'enable_button_icon' => 'yes'
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-criptopayer-elementor-btn-icon' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-criptopayer-elementor-btn-icon i' => 'font-size: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-criptopayer-elementor-btn-icon svg' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};'
                ],
            ]
        );

        $this->add_responsive_control(
            'button_width',
            [
                'label' => esc_html__( 'Width', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-criptopayer-elementor-button-wrapper' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'button_rotate',
            [
                'label' => esc_html__( 'Rotate', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'deg' ],
                'default' => [ 'unit' => 'deg', 'size' => 0, ],
                'range' => [
                    'deg' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-criptopayer-elementor-donation-button' => 'transform: rotate({{SIZE}}deg);',
                ],
            ]
        );

        $this->add_responsive_control(
            'button_offset_x',
            [
                'label' => esc_html__( 'Offset X', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'default' => [ 'unit' => 'px', 'size' => 0, ],
                'range' => [
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-criptopayer-elementor-button-wrapper' => 'transform: translateX({{SIZE}}px);'
                ],
            ]
        );

        $this->add_responsive_control(
            'button_offset_y',
            [
                'label' => esc_html__( 'Offset Y', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'default' => [ 'unit' => 'px', 'size' => 0, ],
                'range' => [
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-criptopayer-elementor-button-transform' => 'transform: translateY({{SIZE}}px);'
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'section_style_button_typography',
                'label' => esc_html__( 'Typography', 'criptopayer-elementor' ),
                'scheme' => Typography::TYPOGRAPHY_1,
                'selector' => "{{WRAPPER}} .mdp-criptopayer-elementor-donation-button",
            ]
        );

        $this->start_controls_tabs( '_style_tabs' );

        $this->start_controls_tab(
            'section_style_button_normal_style_tab',
                  ['label' => esc_html__( 'NORMAL', 'criptopayer-elementor' )]
        );

        $this->add_control(
            'normal_enable_multigradient',
            [
                'label' => esc_html__( 'Enable multigradient background', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'criptopayer-elementor' ),
                'label_off' => esc_html__( 'No', 'criptopayer-elementor' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );


        $this->add_control(
            'section_style_button_normal_text_color',
            [
                'label' => esc_html__( 'Color', 'criptopayer-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-criptopayer-elementor-donation-button" => 'color: {{VALUE}} !important;',
                ],
            ]
        );

        $this->add_control(
            'section_style_button_normal_icon_color',
            [
                'label' => esc_html__( 'Icon color', 'criptopayer-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-criptopayer-elementor-btn-icon" => 'color: {{VALUE}} !important;',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'section_style_button_normal_background',
                'label' => esc_html__( 'Background type', 'criptopayer-elementor' ),
                'types' => ['classic', 'gradient' ],
                'selector' => "{{WRAPPER}} .mdp-criptopayer-elementor-donation-button",
                'condition' => [
                    'normal_enable_multigradient!' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'section_style_button_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_button_border_normal',
                'label' => esc_html__( 'Border Type', 'criptopayer-elementor' ),
                'selector' => "{{WRAPPER}} .mdp-criptopayer-elementor-donation-button",
            ]
        );

        $this->add_responsive_control(
            'section_style_button_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'criptopayer-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-criptopayer-elementor-donation-button" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

       $this->multishadow_controls( 'normal' );

       $this->gradient_controls( 'normal' );

        $this->end_controls_tab();

        $this->start_controls_tab(
            'section_style_button_hover_style_tab',
                  ['label' => esc_html__( 'HOVER', 'criptopayer-elementor' )]
        );

        $this->add_control(
            'hover_enable_multigradient',
            [
                'label' => esc_html__( 'Enable multigradient background', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'criptopayer-elementor' ),
                'label_off' => esc_html__( 'No', 'criptopayer-elementor' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );


        $this->add_control(
            'section_style_button_hover_color',
            [
                'label' => esc_html__( 'Color', 'criptopayer-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-criptopayer-elementor-donation-button:hover" => 'color: {{VALUE}} !important;',
                ],
            ]
        );


        $this->add_control(
            'section_style_button_hover_icon_color',
            [
                'label' => esc_html__( 'Icon color', 'criptopayer-elementor' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-criptopayer-elementor-donation-button:hover .mdp-criptopayer-elementor-btn-icon" => 'color: {{VALUE}} !important;',
                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'section_style_button_background_hover',
                'label' => esc_html__( 'Background type', 'criptopayer-elementor' ),
                'types' => ['classic', 'gradient' ],
                'selector' => "{{WRAPPER}} .mdp-criptopayer-elementor-donation-button:hover",
                'condition' => [
                    'hover_enable_multigradient!' => 'yes'
                ]
            ]
        );


        $this->add_control(
            'section_style_button_hover_transition',
            [
                'label' => esc_html__( 'Hover transition duration', 'criptopayer-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-criptopayer-elementor-donation-button, {{WRAPPER}} .mdp-criptopayer-elementor-btn-icon' => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'section_style_button_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_button_border_hover',
                'label' => esc_html__( 'Border Type', 'criptopayer-elementor' ),
                'selector' => "{{WRAPPER}} .mdp-criptopayer-elementor-donation-button:hover",
            ]
        );

        $this->add_responsive_control(
            'section_style_button_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'criptopayer-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-criptopayer-elementor-donation-button:hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->multishadow_controls( 'hover' );

        $this->gradient_controls( 'hover' );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();

    }


    /**
     * Create warning message if api key is empty.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_api_key_warning_message( $api_key ) {
        return Plugin::$instance->editor->is_edit_mode() && $api_key === '' ?
            sprintf( '<div>%s</div>', esc_html__( 'API Key must not be empty. Please set api key!', 'criptopayer-elementor' ) ) :
            '';
    }

    /**
     * Create donation button icon.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_button_icon( $settings ) {

        if ( empty( $settings['button_icon'] ) ) { return ''; }

        return $settings['button_icon']['library'] === 'svg' ?
            Icons_Manager::render_uploaded_svg_icon( $settings['button_icon']['value'] ) :
            '<i class="'.esc_attr( $settings['button_icon']['value'] ).'"></i>';
    }


    /**
     * Create donation button method.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_donation_button( $settings, $options ) {

        $content = sprintf(
                '<span class="mdp-criptopayer-elementor-content">%s</span>',
                        esc_html( $settings['button_content'] )
        );

        $icon_wrapper = sprintf(
         '<span class="mdp-criptopayer-elementor-btn-icon">%s</span>',
                $this->create_button_icon( $settings )
        );

        return sprintf(
            '<div class="mdp-criptopayer-elementor-button-wrapper">
                    <div class="elementor-animation-%s">
                    <div class="mdp-criptopayer-elementor-button-transform">
                    <a class="mdp-criptopayer-elementor-donation-button 
                                " 
                        href="https://nowpayments.io/donation?api_key=%s" target="_blank">
                       <span class="mdp-criptopayer-elementor-content-wrapper mdp-criptopayer-elementor-icon-position-%s">
                        %s %s
                       </span>
                    </a>
                    </div>
                    </div>
                    </div>',
     esc_attr( $settings['button_hover_animation'] ),
            esc_attr( $options['nowpayments_api_key'] ),
            esc_attr( $settings['button_icon_position'] ),
            $settings['enable_button_icon'] === 'yes' ? $icon_wrapper : '',
            $content
        );
    }
        /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();
        $options = Settings::get_instance()->options;
        $button = $this->create_donation_button( $settings, $options );


        /** Render css styles for widget */
        $this->render_css( $settings );

        echo sprintf( '<!-- Start Criptopayer for Elementor WordPress Plugin -->
                             <div class="mdp-criptopayer-elementor-box mdp-criptopayer-elementor-wrapper-%s">%s</div> %s
                             <!-- End Criptopayer for Elementor WordPress Plugin -->',
                            esc_attr( $this->get_id() ),
                            $button,
                            $this->create_api_key_warning_message( $options['nowpayments_api_key'] )

        );

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/criptopayer';

    }

}
